@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: axis.c,v 1.223.2.14 2017/10/31 00:25:05 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - axis.c */
 
 /*[
@@ -142,6 +138,7 @@ struct lp_style_type grid_lp   = DEFAULT_GRID_LP;
 struct lp_style_type mgrid_lp  = DEFAULT_GRID_LP;
 int grid_layer = LAYER_BEHIND;
 TBOOLEAN grid_tics_in_front = FALSE;
+TBOOLEAN grid_vertical_lines = FALSE;
 double polar_grid_angle = 0;	/* nonzero means a polar grid */
 TBOOLEAN raxis = FALSE;
 double theta_origin = 0.0;	/* default origin at right side */
@@ -215,12 +212,12 @@ axis_invert_if_requested(struct axis *axis)
 
 
 void
-axis_init(AXIS *this_axis, TBOOLEAN infinite)
+axis_init(AXIS *this_axis, TBOOLEAN reset_autoscale)
 {
     this_axis->autoscale = this_axis->set_autoscale;
-    this_axis->min = (infinite && (this_axis->set_autoscale & AUTOSCALE_MIN))
+    this_axis->min = (reset_autoscale && (this_axis->set_autoscale & AUTOSCALE_MIN))
 	? VERYLARGE : this_axis->set_min;
-    this_axis->max = (infinite && (this_axis->set_autoscale & AUTOSCALE_MAX))
+    this_axis->max = (reset_autoscale && (this_axis->set_autoscale & AUTOSCALE_MAX))
 	? -VERYLARGE : this_axis->set_max;
     this_axis->data_min = VERYLARGE;
     this_axis->data_max = -VERYLARGE;
@@ -269,7 +266,7 @@ axis_name(AXIS_INDEX axis)
 	return "t";
 
     if (axis >= PARALLEL_AXES) {
-	sprintf(name, "paxis %d ", axis-PARALLEL_AXES+1);
+	sprintf(name, "paxis %d ", (axis-PARALLEL_AXES+1) & 0xff);
 	return name;
     }
     if (axis < 0) {
@@ -1236,7 +1233,7 @@ gen_tics(struct axis *this, tic_callback callback)
 		    ministart = ministep = step / this->mtic_freq;
 		    miniend = step;
  		}
-	    } else if (nonlinear(this) && this->log) {
+	    } else if (nonlinear(this) && this->ticdef.logscaling) {
 		/* FIXME: Not sure this works for all values of step */
 		ministart = ministep = step / (this->base - 1);
 		miniend = step;
@@ -1766,9 +1763,10 @@ get_num_or_time(struct axis *axis)
     &&  (axis->datatype == DT_TIMEDATE) && isstringvalue(c_token)) {
 	struct tm tm;
 	double usec;
-	char *ss = try_to_get_string();
-	if (gstrptime(ss, timefmt, &tm,&usec, &value) == DT_TIMEDATE)
-	    value = (double) gtimegm(&tm) + usec;
+	char *ss;
+	if ((ss = try_to_get_string()))
+	    if (gstrptime(ss, timefmt, &tm,&usec, &value) == DT_TIMEDATE)
+		value = (double) gtimegm(&tm) + usec;
 	free(ss);
     } else {
 	value = real_expression();
@@ -2406,9 +2404,7 @@ clone_linked_axes(AXIS *axis1, AXIS *axis2)
     double testmin, testmax, scale;
     TBOOLEAN suspect = FALSE;
 
-    /* DEBUG: This sanity check is only here for debugging */
     if (axis1 != axis2->linked_to_primary && axis1 != axis2->linked_to_secondary) {
-	fprintf(stderr, "clone_linked_axes called for axes that are not linked\n");
 	/* No linkage, so nothing to do here */
 	return;
     }
@@ -2485,6 +2481,31 @@ eval_link_function(struct axis *axis, double raw_coord)
     int dummy_var;
     struct value a;
 
+    /* A test for if (undefined) is allowed only immediately following
+     * either evalute_at() or eval_link_function().  Both must clear it
+     * on entry so that the value on return reflects what really happened.
+     */
+    undefined = FALSE;
+
+#if defined(NONLINEAR_AXES) && (NONLINEAR_AXES > 0)
+    /* Special case to speed up evaluation of log scaling
+     * benchmark timing results
+     * v4.6 (old-style logscale)	42.7 u 42.7 total
+     * v5.1 (generic nonlinear)	57.5 u 66.2 total
+     * v5.1 (optimized nonlinear)	42.1 u 42.2 total
+     */
+    if (axis->log) {
+	if (axis->linked_to_secondary) {
+	    if (raw_coord <= 0)
+		return not_a_number();
+	    else
+		return log(raw_coord) / axis->log_base;
+	} else if (axis->linked_to_primary) {
+	    return exp(raw_coord * axis->log_base);
+	}
+    }
+#endif
+
     /* This handles the case "set link x2" with no via/inverse mapping */
     if (link_udf == NULL || link_udf->at == NULL)
 	return raw_coord;
@@ -2544,6 +2565,7 @@ get_shadow_axis(AXIS *axis)
  * This is necessary if we are to reproduce the old logscaling.
  * Extend the tic range on an independent log-scaled axis to the
  * nearest power of 10.
+ * Transfer the new limits over to the user-visible secondary axis.
  */
 void
 extend_primary_ticrange(AXIS *axis)
@@ -2553,11 +2575,15 @@ extend_primary_ticrange(AXIS *axis)
     if (axis->ticdef.logscaling) {
 	/* NB: "zero" is the minimum non-zero value from "set zero" */
 	if ((primary->autoscale & AUTOSCALE_MIN)
-	||  fabs(primary->min - floor(primary->min)) < zero)
+	||  fabs(primary->min - floor(primary->min)) < zero) {
 	    primary->min = floor(primary->min);
+	    axis->min = eval_link_function(axis, primary->min);
+	}
 	if ((primary->autoscale & AUTOSCALE_MAX)
-	||  fabs(primary->max - ceil(primary->max)) < zero)
+	||  fabs(primary->max - ceil(primary->max)) < zero) {
 	    primary->max = ceil(primary->max);
+	    axis->max = eval_link_function(axis, primary->max);
+	}
     }
 }
 
