@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: command.c,v 1.354.2.8 2017/11/04 21:01:45 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - command.c */
 
 /*[
@@ -732,18 +728,20 @@ lower_command(void)
 
 /*
  * Arrays are declared using the syntax
- *    array A[size]
+ *    array A[size] { = [ element, element, ... ] }
  * where size is an integer and space is reserved for elements A[1] through A[size]
- * The size itself is stored in A[0].v.int_val.
+ * The size itself is stored in A[0].v.int_val.A
+ * The list of initial values is optional.
+ * Any element that is not initialized is set to NOTDEFINED.
  *
  * Elements in an existing array can be accessed like any other gnuplot variable.
  * Each element can be one of INTGR, CMPLX, STRING.
- * When the array is declared all elements are set to NOTDEFINED.
  */
 void
 array_command()
 {
-    int nsize = 0;
+    int nsize = 0;	/* Size of array when we leave */
+    int est_size = 0;	/* Estimated size */
     struct udvt_entry *array;
     struct value *A;
     int i;
@@ -754,12 +752,24 @@ array_command()
     array = add_udv(c_token);
     gpfree_array(&array->udv_value);
     gpfree_string(&array->udv_value);
-
-    if (!equals(++c_token,"["))
-	int_error(c_token, "expecting array[size]");
     c_token++;
-    nsize = int_expression();
-    if (!equals(c_token++,"]") || nsize <= 0)
+
+    if (equals(c_token, "[")) {
+	c_token++;
+	nsize = int_expression();
+	if (!equals(c_token++,"]"))
+	    int_error(c_token-1, "expecting array[size>0]");
+    } else if (equals(c_token, "=") && equals(c_token+1, "[")) {
+	/* Estimate size of array by counting commas in the initializer */
+	for ( i = c_token+2; i < num_tokens; i++) {
+	    if (equals(i,",") || equals(i,"]"))
+		est_size++;
+	    if (equals(i,"]"))
+		break;
+	}
+	nsize = est_size;
+    }
+    if (nsize <= 0)
 	int_error(c_token-1, "expecting array[size>0]");
 
     array->udv_value.v.value_array = gp_alloc((nsize+1) * sizeof(t_value), "array_command");
@@ -774,6 +784,7 @@ array_command()
 
     /* Initializer syntax:   array A[10] = [x,y,z,,"foo",] */
     if (equals(c_token, "=")) {
+	int initializers = 0;
 	if (!equals(++c_token, "["))
 	    int_error(c_token, "expecting Array[size] = [x,y,...]");
 	c_token++;
@@ -781,10 +792,12 @@ array_command()
 	    if (equals(c_token, "]"))
 		break;
 	    if (equals(c_token, ",")) {
+		initializers++;
 		c_token++;
 		continue;
 	    }
 	    const_express(&A[i]);
+	    initializers++;
 	    if (equals(c_token, "]"))
 		break;
 	    if (equals(c_token, ","))
@@ -793,6 +806,9 @@ array_command()
 		int_error(c_token, "expecting Array[size] = [x,y,...]");
 	}
 	c_token++;
+	/* If the size is determined by the number of initializers */
+	if (A[0].v.int_val == 0)
+	    A[0].v.int_val = initializers;
     }
 
     return;
@@ -806,7 +822,7 @@ array_command()
 TBOOLEAN
 is_array_assignment()
 {
-    udvt_entry *udv = add_udv(c_token);
+    udvt_entry *udv;
     struct value newvalue;
     int index;
     TBOOLEAN looks_OK = FALSE;
@@ -836,6 +852,7 @@ is_array_assignment()
     if (!looks_OK)
 	return FALSE;
 
+    udv = add_udv(c_token);
     if (udv->udv_value.type != ARRAY)
 	int_error(c_token, "Not a known array");
 
@@ -1034,6 +1051,13 @@ exit_command()
     if (equals(c_token+1,"gnuplot"))
 	gp_exit(EXIT_SUCCESS);
 
+    if (equals(c_token+1,"status")) {
+	int status;
+	c_token += 2;
+	status = int_expression();
+	gp_exit(status);
+    }	
+
     /* exit error 'error message'  returns to the top command line */
     if (equals(c_token+1,"error")) {
 	c_token += 2;
@@ -2030,7 +2054,7 @@ refresh_command()
 void
 refresh_request()
 {
-    FPRINTF((stderr,"refresh_request\n"));
+    AXIS_INDEX axis;
 
     if (   ((first_plot == NULL) && (refresh_ok == E_REFRESH_OK_2D))
         || ((first_3dplot == NULL) && (refresh_ok == E_REFRESH_OK_3D))
@@ -2044,30 +2068,24 @@ refresh_request()
 	return;
     }
 
-    /* If the state has been reset to autoscale since the last plot,
-     * initialize the axis limits.
+    /* Restore the axis range/scaling state from original plot
+     * Dima Kogan April 2018
      */
-    AXIS_INIT2D_REFRESH(FIRST_X_AXIS,TRUE);
-    AXIS_INIT2D_REFRESH(FIRST_Y_AXIS,TRUE);
-    AXIS_INIT2D_REFRESH(SECOND_X_AXIS,TRUE);
-    AXIS_INIT2D_REFRESH(SECOND_Y_AXIS,TRUE);
+    for (axis = 0; axis < NUMBER_OF_MAIN_VISIBLE_AXES; axis++) {
+	AXIS *this_axis = &axis_array[axis];
+	if (this_axis->set_autoscale & AUTOSCALE_MIN)
+	    this_axis->set_min = this_axis->writeback_min;
+	else
+	    this_axis->min = this_axis->set_min;
+	if (this_axis->set_autoscale & AUTOSCALE_MAX)
+	    this_axis->set_max = this_axis->writeback_max;
+	else
+	    this_axis->max = this_axis->set_max;
 
-    AXIS_UPDATE2D_REFRESH(T_AXIS);  /* Untested: T and R want INIT2D or UPDATE2D?? */
-    AXIS_UPDATE2D_REFRESH(POLAR_AXIS);
-
-    AXIS_UPDATE2D_REFRESH(FIRST_Z_AXIS);
-    AXIS_UPDATE2D_REFRESH(COLOR_AXIS);
-
-    /* Nonlinear mapping of x or y via linkage to a hidden primary axis */
-    if (nonlinear(&axis_array[FIRST_X_AXIS])) {
-	AXIS *primary = axis_array[FIRST_X_AXIS].linked_to_primary;
-	primary->min = primary->set_min;
-	primary->max = primary->set_max;
-    }
-    if (nonlinear(&axis_array[FIRST_Y_AXIS])) {
-	AXIS *primary = axis_array[FIRST_Y_AXIS].linked_to_primary;
-	primary->min = primary->set_min;
-	primary->max = primary->set_max;
+	if (this_axis->linked_to_secondary)
+	    clone_linked_axes(this_axis, this_axis->linked_to_secondary);
+	else if (this_axis->linked_to_primary)
+	    clone_linked_axes(this_axis, this_axis->linked_to_primary);
     }
 
     if (refresh_ok == E_REFRESH_OK_2D) {
@@ -2665,13 +2683,28 @@ static float splot_map_surface_rot_x;
 static float splot_map_surface_rot_z;
 static float splot_map_surface_scale;
 
+/* utility routine for splot_map_activate/deactivate */
+static void
+flip_y_axis()
+{
+    struct axis *yaxis = &axis_array[FIRST_Y_AXIS];
+    double temp = yaxis->min;
+    yaxis->min = yaxis->max;
+    yaxis->max = temp;
+    if (yaxis->linked_to_primary) {
+	yaxis = yaxis->linked_to_primary;
+	temp = yaxis->min;
+	yaxis->min = yaxis->max;
+	yaxis->max = temp;
+    }
+}
+
 /* This routine is called at the beginning of 'splot'. It sets up some splot
  * parameters needed to present the 'set view map'.
  */
 void
 splot_map_activate()
 {
-    double temp;
     if (splot_map_active)
 	return;
     splot_map_active = 1;
@@ -2685,9 +2718,7 @@ splot_map_activate()
     /* version 4 had constant value surface_scale = 1.3 */
     surface_scale = 1.425 * mapview_scale;
     /* The Y axis runs backwards from a normal 2D plot */
-    temp = axis_array[FIRST_Y_AXIS].min;
-    axis_array[FIRST_Y_AXIS].min = axis_array[FIRST_Y_AXIS].max;
-    axis_array[FIRST_Y_AXIS].max = temp;
+    flip_y_axis();
 }
 
 
@@ -2697,7 +2728,6 @@ splot_map_activate()
 void
 splot_map_deactivate()
 {
-    double temp;
     if (!splot_map_active)
 	return;
     splot_map_active = 0;
@@ -2706,9 +2736,7 @@ splot_map_deactivate()
     surface_rot_z = splot_map_surface_rot_z;
     surface_scale = splot_map_surface_scale;
     /* The Y axis runs backwards from a normal 2D plot */
-    temp = axis_array[FIRST_Y_AXIS].min;
-    axis_array[FIRST_Y_AXIS].min = axis_array[FIRST_Y_AXIS].max;
-    axis_array[FIRST_Y_AXIS].max = temp;
+    flip_y_axis();
 }
 
 
@@ -3472,8 +3500,8 @@ string_expand_macros()
 	    int_error(NO_CARET, "Macros nested too deeply");
 }
 
-#define COPY_CHAR gp_input_line[o++] = *c; \
-                  after_backslash = FALSE;
+#define COPY_CHAR do { gp_input_line[o++] = *c; \
+                  after_backslash = FALSE; } while (0)
 int
 expand_1level_macros()
 {
