@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: hidden3d.c,v 1.118 2017/03/05 05:55:25 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - hidden3d.c */
 
 /*[
@@ -1051,7 +1047,7 @@ build_networks(struct surface_points *plots, int pcount)
 	long int crvlen;
 	
 	/* Quietly skip empty plots */
-	if (this_plot->plot_type == NODATA)
+	if (this_plot->plot_type == NODATA || this_plot->plot_type == KEYENTRY)
 	    continue;
 
 	crvlen = this_plot->iso_crvs->p_count;
@@ -1163,7 +1159,7 @@ build_networks(struct surface_points *plots, int pcount)
 	lp = &(this_plot->lp_properties);
 
 	/* Quietly skip empty plots */
-	if (this_plot->plot_type == NODATA)
+	if (this_plot->plot_type == NODATA || this_plot->plot_type == KEYENTRY)
 	    continue;
 
 	/* Allow individual plots to opt out of hidden3d calculations */
