@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: parse.c,v 1.112.2.5 2017/09/20 01:40:35 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - parse.c */
 
 /*[
@@ -216,6 +212,10 @@ string_or_express(struct at_type **atptr)
     if (equals(c_token,"$"))
 	return parse_datablock_name();
 
+    /* special keywords */
+    if (equals(c_token,"keyentry"))
+	return NULL;
+
     if (isstring(c_token) && (str = try_to_get_string()))
 	return str;
 
@@ -491,12 +491,15 @@ parse_array_assignment_expression()
 
 	/* Quick check for the most common false positives */
 	/* i.e. other constructs that begin with "name["   */
-	if (equals(c_token,"sum") && equals(c_token+3, "="))
-	    return 0;
-	if (equals(c_token+3, ":"))
-	    return 0;
+	/* FIXME: quicker than the full test below, but do we care? */
 	if (equals(c_token+3, "]") && !equals(c_token+4, "="))
 	    return 0;
+	if (equals(c_token+3, ":"))	/* substring s[foo:baz] */
+	    return 0;
+
+	/* Is this really a known array name? */
+	if (type_udv(c_token) != ARRAY)
+	    return 0;
 
 	/* Save state of the action table and the command line */
 	save_action = at->a_count;
