@@ -68,6 +68,7 @@ extern "C" {
 	#include "win/wtext.h"    // for kbhit, getchar
 	#include <io.h>           // for isatty
 #endif
+	#include "readline.h"	// for wrap_readline_signal_handler() to catch ^Z
 	#include <signal.h>
 }
 
@@ -418,6 +419,11 @@ bool qt_processTermEvent(gp_event_t* event)
 		paused_for_mouse = 0;
 		return true;
 	}
+	if ((event->type == GE_reset))
+	{
+		paused_for_mouse = 0;
+		return true;
+	}
 
 	return false;
 }
@@ -1024,6 +1030,10 @@ int qt_waitforinput(int options)
 						timeout );
 
 		if (n_changed_fds < 0) {
+			if (errno == EINTR) {
+				wrap_readline_signal_handler();
+				continue;
+			}
 			// Display the error message except when Ctrl + C is pressed
 			if (errno != 4)
 				fprintf(stderr, "Qt terminal communication error: select() error %i %s\n", errno, strerror(errno));
