@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: save.c,v 1.328.2.1 2017/10/10 03:55:43 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - save.c */
 
 /*[
@@ -138,11 +134,13 @@ save_variables__sub(FILE *fp)
 
     while (udv) {
 	if (udv->udv_value.type != NOTDEFINED) {
-	    if (udv->udv_value.type == ARRAY) {
-		fprintf(fp,"array %s[%d] = ", udv->udv_name,
+	    if ((udv->udv_value.type == ARRAY)
+		&& strncmp(udv->udv_name,"ARGV",4)) {
+		    fprintf(fp,"array %s[%d] = ", udv->udv_name,
 			udv->udv_value.v.value_array[0].v.int_val);
-		save_array_content(fp, udv->udv_value.v.value_array);
+		    save_array_content(fp, udv->udv_value.v.value_array);
 	    } else if (strncmp(udv->udv_name,"GPVAL_",6)
+		 && strncmp(udv->udv_name,"GPFUN_",6)
 		 && strncmp(udv->udv_name,"MOUSE_",6)
 		 && strncmp(udv->udv_name,"$",1)
 		 && (strncmp(udv->udv_name,"ARG",3) || (strlen(udv->udv_name) != 4))
@@ -379,7 +377,9 @@ save_set_all(FILE *fp)
 	fputs("\n", fp);
 #undef SAVE_GRID
 
-	fprintf(fp, "set grid %s  ", (grid_layer==-1) ? "layerdefault" : ((grid_layer==0) ? "back" : "front"));
+	fprintf(fp, "set grid %s%s  ",
+		(grid_vertical_lines) ? "vertical " : "",
+		(grid_layer==-1) ? "layerdefault" : ((grid_layer==0) ? "back" : "front"));
 	save_linetype(fp, &grid_lp, FALSE);
 	fprintf(fp, ", ");
 	save_linetype(fp, &mgrid_lp, FALSE);
@@ -712,10 +712,10 @@ set isosamples %d, %d\n\
     fputs("set cntrparam levels ", fp);
     switch (contour_levels_kind) {
     case LEVELS_AUTO:
-	fprintf(fp, "auto %d\n", contour_levels);
+	fprintf(fp, "auto %d", contour_levels);
 	break;
     case LEVELS_INCREMENTAL:
-	fprintf(fp, "incremental %g,%g,%g\n",
+	fprintf(fp, "incremental %g,%g,%g",
 		contour_levels_list[0], contour_levels_list[1],
 		contour_levels_list[0] + contour_levels_list[1] * contour_levels);
 	break;
@@ -725,9 +725,10 @@ set isosamples %d, %d\n\
 	    fprintf(fp, "discrete %g", contour_levels_list[0]);
 	    for (i = 1; i < contour_levels; i++)
 		fprintf(fp, ",%g ", contour_levels_list[i]);
-	    fputc('\n', fp);
 	}
     }
+    fprintf(fp, " %ssorted\n", contour_sortlevels ? "" : "un");
+    fprintf(fp, "set cntrparam firstlinetype %d\n", contour_firstlinetype);
     fprintf(fp, "\
 set cntrparam points %d\n\
 set size ratio %g %g,%g\n\
@@ -874,7 +875,7 @@ set origin %g,%g\n",
     case PM3D_SCANS_AUTOMATIC: fputs("scansautomatic\n", fp); break;
     case PM3D_SCANS_FORWARD: fputs("scansforward\n", fp); break;
     case PM3D_SCANS_BACKWARD: fputs("scansbackward\n", fp); break;
-    case PM3D_DEPTH: fputs("depthorder\n", fp); break;
+    case PM3D_DEPTH: fprintf(fp, "depthorder %s\n", pm3d.base_sort ? "base" : ""); break;
     }
     fprintf(fp, "set pm3d interpolate %d,%d", pm3d.interp_i, pm3d.interp_j);
     fputs(" flush ", fp);
@@ -930,8 +931,6 @@ set origin %g,%g\n",
 	case C_MODEL_RGB: fputs( "RGB ", fp ); break;
 	case C_MODEL_HSV: fputs( "HSV ", fp ); break;
 	case C_MODEL_CMY: fputs( "CMY ", fp ); break;
-	case C_MODEL_YIQ: fputs( "YIQ ", fp ); break;
-	case C_MODEL_XYZ: fputs( "XYZ ", fp ); break;
       }
       fputs( "\nset palette ", fp );
       switch( sm_palette.colorMode ) {
