@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: tables.c,v 1.155.2.1 2017/10/10 03:55:45 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - tables.c */
 
 /*[
@@ -221,9 +217,7 @@ const struct gen_table set_tbl[] =
 
     { "micro", S_MICRO },
     { "minus$sign", S_MINUS_SIGN },
-#ifdef USE_MOUSE
     { "mo$use", S_MOUSE },
-#endif
     { "mono$chrome", S_MONOCHROME },
     { "multi$plot", S_MULTIPLOT },
 
@@ -486,8 +480,7 @@ const struct gen_table color_model_tbl[] =
     { "RGB", C_MODEL_RGB },
     { "HSV", C_MODEL_HSV },
     { "CMY", C_MODEL_CMY },
-    { "YIQ", C_MODEL_YIQ },
-    { "XYZ", C_MODEL_XYZ },
+    { "XYZ", C_MODEL_XYZ },	/* Obsolete */
     { NULL,  -1 }
 };
 
