@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: term.c,v 1.332.2.4 2017/07/08 23:49:45 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - term.c */
 
 /*[
@@ -93,6 +89,7 @@ static char *RCSid() { return RCSid("$Id: term.c,v 1.332.2.4 2017/07/08 23:49:45
 #include "version.h"
 #include "misc.h"
 #include "multiplot.h"
+#include "readline.h"
 
 #ifdef USE_MOUSE
 #include "mouse.h"
@@ -498,6 +495,14 @@ term_initialise()
 	FPRINTF((stderr, "- calling term->init()\n"));
 	(*term->init) ();
 	term_initialised = TRUE;
+#ifdef HAVE_LOCALE_H
+	/* This is here only from an abundance of caution (a.k.a. paranoia).
+	 * Some terminals (wxt qt caca) are known to change the locale when
+	 * initialized.  Others have been implicated (gd).  Rather than trying
+	 * to catch all such offenders one by one, cover for all of them here.
+	 */
+	setlocale(LC_NUMERIC, "C");
+#endif
     }
 }
 
@@ -1129,7 +1134,7 @@ do_arrow(
 
 void
 do_arc(
-    unsigned int cx, unsigned int cy, /* Center */
+    int cx, int cy, /* Center */
     double radius, /* Radius */
     double arc_start, double arc_end, /* Limits of arc in degress */
     int style, TBOOLEAN wedge)
@@ -2602,7 +2607,7 @@ enh_err_check(const char *str)
  * developed later they can be slotted into this one call site.
  */
 int
-estimate_strlen(char *text)
+estimate_strlen(const char *text)
 {
 int len;
 
@@ -2644,11 +2649,17 @@ estimate_plaintext(char *enhancedtext)
 void
 ignore_enhanced(TBOOLEAN flag)
 {
-    /* Force a return to the default font */
+#if (0)
+    /* Apr 2018: This code was introduced long ago (2005; Bug #266) to address
+     * a glitch in the postscript terminal that left the last-used font in
+     * an enhanced text string active afterwards.
+     * We now deal with this in ENHPS_put_text() instead.
+     */
     if (flag && !ignore_enhanced_text) {
 	ignore_enhanced_text = TRUE;
 	term->set_font("");
     }
+#endif
     ignore_enhanced_text = flag;
 }
 
@@ -2945,7 +2956,7 @@ strlen_tex(const char *str)
 	switch (*s) {
 	case '[':
 		while (*s && *s != ']') s++;
-		s++;
+		if (*s) s++;
 		break;
 	case '\\':
 		s++;
