@@ -1,7 +1,3 @@
-/*
- * $Id: term_api.h,v 1.152 2017/05/07 19:20:52 sfeam Exp $
- */
-
 /* GNUPLOT - term_api.h */
 
 /*[
@@ -487,7 +483,7 @@ void init_monochrome __PROTO((void));
 struct termentry *change_term __PROTO((const char *name, int length));
 
 void write_multiline __PROTO((unsigned int, unsigned int, char *, JUSTIFY, VERT_JUSTIFY, int, const char *));
-int estimate_strlen __PROTO((char *));
+int estimate_strlen __PROTO((const char *));
 char *estimate_plaintext __PROTO((char *));
 void list_terms __PROTO((void));
 char* get_terminals_names __PROTO((void));
@@ -515,7 +511,7 @@ int style_from_fill __PROTO((struct fill_style_type *));
 
 #ifdef EAM_OBJECTS
 /* Terminal-independent routine to draw a circle or arc */
-void do_arc __PROTO(( unsigned int cx, unsigned int cy, double radius,
+void do_arc __PROTO(( int cx, int cy, double radius,
                       double arc_start, double arc_end,
 		      int style, TBOOLEAN wedge));
 #endif
