@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: unset.c,v 1.254.2.2 2017/10/10 03:55:45 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - unset.c */
 
 /*[
@@ -45,6 +41,7 @@ static char *RCSid() { return RCSid("$Id: unset.c,v 1.254.2.2 2017/10/10 03:55:4
 #include "hidden3d.h"
 #include "jitter.h"
 #include "misc.h"
+#include "multiplot.h"
 #include "parse.h"
 #include "plot.h"
 #include "plot2d.h"
@@ -107,9 +104,7 @@ static void unset_margin __PROTO((t_position *));
 static void unset_missing __PROTO((void));
 static void unset_micro __PROTO((void));
 static void unset_minus_sign __PROTO((void));
-#ifdef USE_MOUSE
 static void unset_mouse __PROTO((void));
-#endif
 
 static void unset_month_day_tics __PROTO((AXIS_INDEX));
 static void unset_minitics __PROTO((struct axis *));
@@ -344,11 +339,9 @@ unset_command()
     case S_MONOCHROME:
 	unset_monochrome();
 	break;
-#ifdef USE_MOUSE
     case S_MOUSE:
 	unset_mouse();
 	break;
-#endif
     case S_MULTIPLOT:
 	term_end_multiplot();
 	break;
@@ -815,6 +808,8 @@ unset_cntrparam()
     contour_order = DEFAULT_CONTOUR_ORDER;
     contour_levels = DEFAULT_CONTOUR_LEVELS;
     contour_levels_kind = LEVELS_AUTO;
+    contour_firstlinetype = 0;
+    contour_sortlevels = FALSE;
 }
 
 /* process 'unset cntrlabel' command */
@@ -944,6 +939,7 @@ unset_grid()
 	axis_array[i].gridminor = FALSE;
     }
     polar_grid_angle = 0;
+    grid_vertical_lines = FALSE;
 }
 
 
@@ -1259,18 +1255,18 @@ unset_missing()
     missing_val = NULL;
 }
 
-#ifdef USE_MOUSE
 /* process 'unset mouse' command */
 static void
 unset_mouse()
 {
+#ifdef USE_MOUSE
     mouse_setting.on = 0;
 #ifdef OS2
     PM_update_menu_items();
 #endif
     UpdateStatusline(); /* wipe status line */
-}
 #endif
+}
 
 /* process 'unset mxtics' command */
 static void
@@ -1436,7 +1432,6 @@ unset_pm3d()
     /* reset styles, required to 'plot something' after e.g. 'set pm3d map' */
     if (data_style == PM3DSURFACE) data_style = POINTSTYLE;
     if (func_style == PM3DSURFACE) func_style = LINES;
-    pm3d.border.l_type = LT_NODRAW;
 }
 
 
@@ -1999,6 +1994,10 @@ reset_command()
     mouse_setting = default_mouse_setting;
 #endif
 
+    /* restore previous multiplot offset and margins */
+    if (multiplot)
+	multiplot_reset();
+
     unset_missing();
     free(df_separators);
     df_separators = NULL;
