@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: util.c,v 1.147.2.3 2017/08/30 18:37:07 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - util.c */
 
 /*[
@@ -194,6 +190,22 @@ isletter(int t_num)
 	    (isalpha(c) || (c == '_') || ALLOWED_8BITVAR(c)));
 }
 
+/* Test whether following bit of command line might be parsable as a number.
+ * constant, defined variable, function, ...
+ */
+TBOOLEAN
+might_be_numeric(int t_num)
+{
+    if (END_OF_COMMAND)
+	return FALSE;
+    if (isanumber(t_num) || is_function(t_num))
+	return TRUE;
+    if (type_udv(t_num) == INTGR || type_udv(t_num) == CMPLX || type_udv(t_num) == ARRAY)
+	return TRUE;
+    if (equals(t_num, "("))
+	return TRUE;
+    return FALSE;
+}
 
 /*
  * is_definition() returns TRUE if the next tokens are of the form
@@ -1027,6 +1039,7 @@ static void
 print_line_with_error(int t_num)
 {
     int i;
+    int true_line_num = inline_num;
 
     if (t_num == DATAFILE) {
 	/* Print problem line from data file to the terminal */
@@ -1036,17 +1049,19 @@ print_line_with_error(int t_num)
 
 	/* If the current line was built by concatenation of lines inside */
 	/* a {bracketed clause}, try to reconstruct the true line number  */
-	char *minimal_input_line = gp_input_line;
+	/* FIXME:  This seems to no longer work reliably */
+	char *copy_of_input_line = gp_strdup(gp_input_line);
+	char *minimal_input_line = copy_of_input_line;
 	char *trunc;
-	while ((trunc = strrchr(gp_input_line, '\n')) != NULL) {
+	while ((trunc = strrchr(copy_of_input_line, '\n')) != NULL) {
 	    int current = (t_num == NO_CARET) ? c_token : t_num;
-	    if (trunc < &gp_input_line[token[current].start_index]) {
+	    if (trunc < &copy_of_input_line[token[current].start_index]) {
 		minimal_input_line = trunc+1;
 		t_num = NO_CARET;
 		break;
 	    }
 	    *trunc = '\0';
-	    inline_num--;
+	    true_line_num--;
 	}
 
 	if (t_num != NO_CARET) {
@@ -1065,14 +1080,19 @@ print_line_with_error(int t_num)
 	    /* Print token */
 	    fputs("^\n",stderr);
 	}
+	free(copy_of_input_line);
     }
 
     PRINT_SPACES_UNDER_PROMPT;
 
     if (!interactive) {
-	if (lf_head && lf_head->name)
-	    fprintf(stderr, "\"%s\", ", lf_head->name);
-	fprintf(stderr, "line %d: ", inline_num);
+	LFS *lf = lf_head;
+	/* Back out of any nested if/else clauses */
+	while (lf && !lf->fp && !lf->name && lf->prev)
+	    lf = lf->prev;
+	if (lf && lf->name)
+	    fprintf(stderr, "\"%s\" ", lf->name);
+	fprintf(stderr, "line %d: ", true_line_num);
     }
 }
 
@@ -1175,6 +1195,9 @@ common_error_exit()
     plot_iterator = cleanup_iteration(plot_iterator);
     scanning_range_in_progress = FALSE;
     inside_zoom = FALSE;
+#ifdef HAVE_LOCALE_H
+    setlocale(LC_NUMERIC, "C");
+#endif
 
     /* Load error state variables */
     update_gpval_variables(2);
